Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Windows
Imports System.Windows.Controls
Imports ESRI.ArcGIS.Client
Imports ESRI.ArcGIS.Client.Geometry
Imports ESRI.ArcGIS.Client.Tasks
Imports ESRI.ArcGIS.Client.Symbols


  Partial Public Class ViewShed
    Inherits UserControl
    Private _geoprocessorTask As Geoprocessor

    Public Sub New()
      InitializeComponent()
      _geoprocessorTask = New Geoprocessor("http://sampleserver1.arcgisonline.com/ArcGIS/rest/services/" & "Elevation/ESRI_Elevation_World/GPServer/Viewshed")
      AddHandler _geoprocessorTask.ExecuteCompleted, AddressOf GeoprocessorTask_ExecuteCompleted
      AddHandler _geoprocessorTask.Failed, AddressOf GeoprocessorTask_Failed
    End Sub

    Private Sub MyMap_MouseClick(ByVal sender As Object, ByVal e As ESRI.ArcGIS.Client.Map.MouseEventArgs)
      _geoprocessorTask.CancelAsync()

      Dim graphicsLayer As GraphicsLayer = TryCast(MyMap.Layers("MyGraphicsLayer"), GraphicsLayer)
      graphicsLayer.ClearGraphics()

      Dim mapPoint As MapPoint = e.MapPoint
      mapPoint.SpatialReference = New SpatialReference(4326)

      Dim graphic As New Graphic() With {.Symbol = TryCast(LayoutRoot.Resources("StartMarkerSymbol"), Symbol), .Geometry = mapPoint}
      graphicsLayer.Graphics.Add(graphic)

      MyMap.Cursor = System.Windows.Input.Cursors.Wait

      Dim parameters As New List(Of GPParameter)()
      parameters.Add(New GPFeatureRecordSetLayer("Input_Observation_Point", mapPoint))
      parameters.Add(New GPLinearUnit("Viewshed_Distance", esriUnits.esriMiles, Convert.ToDouble(MilesTextBox.Text)))

      _geoprocessorTask.OutputSpatialReference = New SpatialReference(4326)
      _geoprocessorTask.ExecuteAsync(parameters)
    End Sub

    Private Sub GeoprocessorTask_ExecuteCompleted(ByVal sender As Object, ByVal args As ESRI.ArcGIS.Client.Tasks.GPExecuteCompleteEventArgs)
      MyMap.Cursor = System.Windows.Input.Cursors.Hand
      Dim graphicsLayer As GraphicsLayer = TryCast(MyMap.Layers("MyGraphicsLayer"), GraphicsLayer)

      For Each gpParameter As GPParameter In args.Results.OutParameters
        If TypeOf gpParameter Is GPFeatureRecordSetLayer Then
          Dim layer As GPFeatureRecordSetLayer = TryCast(gpParameter, GPFeatureRecordSetLayer)
          For Each graphic As Graphic In layer.FeatureSet.Features
            graphic.Symbol = TryCast(LayoutRoot.Resources("DefaultFillSymbol"), Symbol)
            graphicsLayer.Graphics.Add(graphic)
          Next graphic
        End If
      Next gpParameter
    End Sub

    Private Sub GeoprocessorTask_Failed(ByVal sender As Object, ByVal e As TaskFailedEventArgs)
      MyMap.Cursor = System.Windows.Input.Cursors.Hand
      MessageBox.Show("Geoprocessor service failed: " & e.Error.Message)
    End Sub
  End Class

