Imports Microsoft.VisualBasic
Imports System.Windows
Imports System.Windows.Controls
Imports ESRI.ArcGIS.Client
Imports ESRI.ArcGIS.Client.Geometry
Imports ESRI.ArcGIS.Client.Tasks
Imports ESRI.ArcGIS.Client.Symbols


	Partial Public Class BufferQuery
		Inherits UserControl
		Private _geometryService As GeometryService
		Private _queryTask As QueryTask

		Public Sub New()
			InitializeComponent()

			_geometryService = New GeometryService("http://tasks.arcgisonline.com/ArcGIS/rest/services/Geometry/GeometryServer")
			AddHandler _geometryService.BufferCompleted, AddressOf GeometryService_BufferCompleted
			AddHandler _geometryService.Failed, AddressOf GeometryService_Failed

			_queryTask = New QueryTask("http://sampleserver3.arcgisonline.com/ArcGIS/rest/services/BloomfieldHillsMichigan/Parcels/MapServer/2")
			AddHandler _queryTask.ExecuteCompleted, AddressOf QueryTask_ExecuteCompleted
			AddHandler _queryTask.Failed, AddressOf QueryTask_Failed
		End Sub

		Private Sub MyMap_MouseClick(ByVal sender As Object, ByVal e As ESRI.ArcGIS.Client.Map.MouseEventArgs)
			_geometryService.CancelAsync()
			_queryTask.CancelAsync()

			Dim clickGraphic As New Graphic()
			clickGraphic.Symbol = TryCast(LayoutRoot.Resources("DefaultMarkerSymbol"), Symbol)
			clickGraphic.Geometry = e.MapPoint
			' Input spatial reference for buffer operation defined by first feature of input geometry array
			clickGraphic.Geometry.SpatialReference = MyMap.SpatialReference

			Dim graphicsLayer As GraphicsLayer = TryCast(MyMap.Layers("MyGraphicsLayer"), GraphicsLayer)
			graphicsLayer.ClearGraphics()

			clickGraphic.SetZIndex(2)
			graphicsLayer.Graphics.Add(clickGraphic)

			' Use a projection appropriate for your area of interest
			Dim bufferParams As New ESRI.ArcGIS.Client.Tasks.BufferParameters() With
				{.BufferSpatialReference = New SpatialReference(4326),
				 .OutSpatialReference = MyMap.SpatialReference,
				 .Unit = LinearUnit.Meter}
			bufferParams.Distances.Add(100)
			bufferParams.Features.Add(clickGraphic)

			_geometryService.BufferAsync(bufferParams)
		End Sub

		Private Sub GeometryService_BufferCompleted(ByVal sender As Object, ByVal args As GraphicsEventArgs)
			Dim bufferGraphic As New Graphic()
			bufferGraphic.Geometry = args.Results(0).Geometry
			bufferGraphic.Symbol = TryCast(LayoutRoot.Resources("BufferSymbol"), Symbol)
			bufferGraphic.SetZIndex(1)

			Dim graphicsLayer As GraphicsLayer = TryCast(MyMap.Layers("MyGraphicsLayer"), GraphicsLayer)
			graphicsLayer.Graphics.Add(bufferGraphic)

			Dim query As New ESRI.ArcGIS.Client.Tasks.Query()
			query.ReturnGeometry = True
			query.Geometry = bufferGraphic.Geometry
			query.OutSpatialReference = MyMap.SpatialReference
			query.OutFields.Add("OWNERNME1")
			_queryTask.ExecuteAsync(query)
		End Sub

		Private Sub QueryTask_ExecuteCompleted(ByVal sender As Object, ByVal args As QueryEventArgs)
			If args.FeatureSet Is Nothing Then
				Return
			End If

			Dim graphicsLayer As GraphicsLayer = TryCast(MyMap.Layers("MyResultsGraphicsLayer"), GraphicsLayer)
			For Each selectedGraphic As Graphic In args.FeatureSet.Features
				selectedGraphic.Symbol = TryCast(LayoutRoot.Resources("ParcelSymbol"), Symbol)
				graphicsLayer.Graphics.Add(selectedGraphic)
			Next selectedGraphic
		End Sub

		Private Sub GeometryService_Failed(ByVal sender As Object, ByVal args As TaskFailedEventArgs)
			MessageBox.Show("Geometry service failed: " & args.Error.Message)
		End Sub

		Private Sub QueryTask_Failed(ByVal sender As Object, ByVal args As TaskFailedEventArgs)
			MessageBox.Show("Query failed: " & args.Error.Message)
		End Sub

	End Class

