﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Input
Imports System.Windows.Media
Imports ESRI.ArcGIS.Client
Imports System.Windows.Media.Imaging
Imports ESRI.ArcGIS.Client.Symbols
Imports System.Web.Script.Serialization
Imports System.IO



	Partial Public Class SOEElevationData
		Inherits UserControl
		Private myDrawObject As Draw
		Private webClient As WebClient
		Private graphicsLayer As GraphicsLayer
		Private colorRanges As New List(Of Color)()
		Private serializer As JavaScriptSerializer

		Public Sub New()
			InitializeComponent()

			serializer = New JavaScriptSerializer()

			myDrawObject = New Draw(MyMap) With {.DrawMode = DrawMode.Rectangle, .FillSymbol = TryCast(LayoutRoot.Resources("DrawFillSymbol"), ESRI.ArcGIS.Client.Symbols.FillSymbol), .IsEnabled = True}
			AddHandler myDrawObject.DrawComplete, AddressOf drawObject_DrawComplete
			AddHandler myDrawObject.DrawBegin, AddressOf drawObject_OnDrawBegin

			webClient = New WebClient()
			AddHandler webClient.OpenReadCompleted, AddressOf wc_OpenReadCompleted

			graphicsLayer = TryCast(MyMap.Layers("MyGraphicsLayer"), GraphicsLayer)

			colorRanges.Add(Colors.Blue)
			colorRanges.Add(Colors.Green)
			colorRanges.Add(Colors.Yellow)
			colorRanges.Add(Colors.Orange)
			colorRanges.Add(Colors.Red)
		End Sub

		Private Sub drawObject_OnDrawBegin(ByVal sender As Object, ByVal args As EventArgs)
			graphicsLayer.ClearGraphics()
			ElevationView.Visibility = Visibility.Collapsed
		End Sub

		Private Sub drawObject_DrawComplete(ByVal sender As Object, ByVal e As DrawEventArgs)
			myDrawObject.IsEnabled = False

			Dim graphic As New ESRI.ArcGIS.Client.Graphic() With {.Geometry = e.Geometry, .Symbol = TryCast(LayoutRoot.Resources("DrawFillSymbol"), Symbol)}
			graphicsLayer.Graphics.Add(graphic)

			Dim aoiEnvelope As ESRI.ArcGIS.Client.Geometry.Envelope = TryCast(e.Geometry, ESRI.ArcGIS.Client.Geometry.Envelope)

			Dim SOEurl As String = "http://sampleserver4.arcgisonline.com/ArcGIS/rest/services/Elevation/ESRI_Elevation_World/MapServer/exts/ElevationsSOE/ElevationLayers/1/GetElevationData?"

			SOEurl &= String.Format(System.Globalization.CultureInfo.InvariantCulture, "Extent={{""xmin"" : {0}, ""ymin"" : {1}, ""xmax"" : {2}, ""ymax"" :{3},""spatialReference"" : {{""wkid"" : {4}}}}}&Rows={5}&Columns={6}&f=json", aoiEnvelope.XMin, aoiEnvelope.YMin, aoiEnvelope.XMax, aoiEnvelope.YMax, MyMap.SpatialReference.WKID, HeightTextBox.Text, WidthTextBox.Text)

			webClient.OpenReadAsync(New Uri(SOEurl))
		End Sub

		Private Sub wc_OpenReadCompleted(ByVal sender As Object, ByVal e As OpenReadCompletedEventArgs)

			Dim reader As New StreamReader(e.Result)
			Dim responseString As String = reader.ReadToEnd()
			Dim results As IDictionary(Of String, Object) = TryCast(serializer.DeserializeObject(responseString), IDictionary(Of String, Object))

			e.Result.Close()

			If results.ContainsKey("error") Then
				MessageBox.Show("Error") 'results["error"]["code"] + ": " + results["error"]["message"]
				myDrawObject.IsEnabled = True
				Return
			End If

			If results.ContainsKey("data") Then
				Dim elevData As IEnumerable(Of Object) = TryCast(results("data"), IEnumerable(Of Object))

				Dim thematicMin, thematicMax As Integer
				thematicMax = CInt(Fix(elevData.ElementAt(0)))
				thematicMin = thematicMax

				For Each elevValue As Integer In elevData
					If elevValue < thematicMin Then
						thematicMin = elevValue
					End If
					If elevValue > thematicMax Then
						thematicMax = elevValue
					End If
				Next elevValue

				Dim totalRange As Integer = thematicMax - thematicMin
				Dim portion As Integer = totalRange \ 5

				Dim cellColor As New List(Of Color)()

				For Each elevValue As Integer In elevData
					Dim startValue As Integer = thematicMin
					For i As Integer = 0 To 4
						If Enumerable.Range(startValue, portion).Contains(elevValue) Then
							cellColor.Add(colorRanges(i))
							Exit For
						ElseIf i = 4 Then
							cellColor.Add(colorRanges.Last())
						End If

						startValue = startValue + portion
					Next i
				Next elevValue

				Dim rows As Integer = Convert.ToInt32(HeightTextBox.Text)
				Dim cols As Integer = Convert.ToInt32(WidthTextBox.Text)
				Dim stride As Integer = cols * 3 + (cols * 3) Mod 4

				Dim cell As Integer = 0
				Dim bits(rows * stride - 1) As Byte
				For x As Integer = 0 To rows - 1
					For y As Integer = 0 To cols - 1
						SetPixel(bits, x, y, stride, cellColor(cell))
						cell += 1
					Next y
				Next x

				Dim bs As BitmapSource = BitmapSource.Create(rows, cols, 300, 300, PixelFormats.Rgb24, Nothing, bits, stride)
				ElevationView.Visibility = System.Windows.Visibility.Visible
				ElevationImage.Source = bs
				myDrawObject.IsEnabled = True
			End If
		End Sub

		Private Sub SetPixel(ByRef bits() As Byte, ByVal x As Integer, ByVal y As Integer, ByVal stride As Integer, ByVal c As Color)
			bits(x * 3 + y * stride) = c.R
			bits(x * 3 + y * stride + 1) = c.G
			bits(x * 3 + y * stride + 2) = c.B
		End Sub

		Private Sub CloseProfileImage_MouseLeftButtonDown(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
			ElevationView.Visibility = Visibility.Collapsed
		End Sub

		Private Sub SizeProfileImage_MouseLeftButtonDown(ByVal sender As Object, ByVal e As MouseButtonEventArgs)
			If ElevationImage.Width = 150 Then
				ElevationImage.Height = 300
				ElevationImage.Width = ElevationImage.Height
			Else
				ElevationImage.Height = 150
				ElevationImage.Width = ElevationImage.Height
			End If
		End Sub
	End Class

